package fixtory

import (
	"bytes"
	"github.com/k-yomo/fixtory/pkg/astutil"
	"go/ast"
	"go/format"
	"os"
	"path/filepath"
	"strings"
	"text/template"
)

type tmplParam struct {
	StructName string
	FieldNames []string
}

func Generate(targetDir string, types []string) error {
	targetTypeMap := map[string]bool{}
	for _, t := range types {
		targetTypeMap[t] = true
	}

	walkerMap, err := astutil.DirToAstWalker(targetDir)
	if err != nil {
		return err
	}
	for _, walker := range walkerMap {
		if len(targetTypeMap) == 0 {
			break
		}
		body := new(bytes.Buffer)
		for _, spec := range walker.AllStructSpecs() {
			if len(targetTypeMap) == 0 {
				break
			}
			if !targetTypeMap[spec.Name.Name] {
				continue
			}
			delete(targetTypeMap, spec.Name.Name)

			structType := spec.Type.(*ast.StructType)
			fieldNames := make([]string, 0, len(structType.Fields.List))
			for _, field := range structType.Fields.List {
				fieldNames = append(fieldNames, field.Names[0].Name)
			}
			tpl := template.Must(template.New("factorytpl.go").Funcs(template.FuncMap{"ToLower": strings.ToLower}).Parse(factoryTpl))
			params := tmplParam{StructName: spec.Name.Name, FieldNames: fieldNames}
			if err := tpl.Execute(body, params); err != nil {
				panic(err)
			}
		}
		if body.Len() == 0 {
			continue
		}

		out := new(bytes.Buffer)
		err = template.Must(template.New("out").Parse(`
			// Code generated by {{ .GeneratorName }}; DO NOT EDIT.
		
			package {{ .PackageName }}

			import (
				"github.com/k-yomo/fixtory"
				"testing"
			)
		
			{{ .Body }}
		`)).Execute(out, map[string]string{
			"GeneratorName": "fixtory",
			"PackageName":   walker.Pkg.Name,
			"Body":          body.String(),
		})
		if err != nil {
			return err
		}

		str, err := format.Source(out.Bytes())
		if err != nil {
			panic(err)
		}

		dstFilePath := filepath.Join(filepath.FromSlash(targetDir), "fixtory_gen.go")
		writer, err := os.Create(dstFilePath)
		if err != nil {
			panic(err)
		}
		defer writer.Close()

		if _, err := writer.Write(str); err != nil {
			return err
		}
	}

	return nil
}
